//                                               -*- C++ -*-
/**
 *  @file  Histogram.hxx
 *  @brief The Histogram distribution
 *
 *  (C) Copyright 2005-2011 EDF-EADS-Phimeca
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License.
 *
 *  This library is distributed in the hope that it will be useful
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 *
 *  @author: $LastChangedBy: souchaud $
 *  @date:   $LastChangedDate: 2011-07-01 10:34:36 +0200 (Fri, 01 Jul 2011) $
 *  Id:      $Id: Histogram.hxx 1981 2011-07-01 08:34:36Z souchaud $
 */
#ifndef OPENTURNS_HISTOGRAM_HXX
#define OPENTURNS_HISTOGRAM_HXX

#include "OTprivate.hxx"
#include "NonEllipticalDistribution.hxx"
#include "Collection.hxx"
#include "HistogramPair.hxx"
#include "PersistentCollection.hxx"
#include "PersistentObject.hxx"

namespace OpenTURNS {

  namespace Uncertainty {

    namespace Distribution {


      /**
       * @class Histogram
       *
       * The Histogram distribution.
       */
      class Histogram
        : public Model::NonEllipticalDistribution
      {
        CLASSNAME;

      public:

        typedef Base::Common::InvalidArgumentException               InvalidArgumentException;
        typedef Model::NonEllipticalDistribution                     NonEllipticalDistribution;  // required by SWIG
        typedef NonEllipticalDistribution::NumericalPoint            NumericalPoint;
        typedef NonEllipticalDistribution::NumericalSample           NumericalSample;
        typedef Base::Type::Collection<HistogramPair>                HistogramPairCollection;
        typedef Base::Type::PersistentCollection<HistogramPair>      HistogramPairPersistentCollection;
        typedef NonEllipticalDistribution::CovarianceMatrix          CovarianceMatrix;
        typedef NonEllipticalDistribution::NumericalPointWithDescriptionCollection  NumericalPointWithDescriptionCollection;
        typedef NonEllipticalDistribution::NotDefinedException       NotDefinedException;

        /** Default constructor */
        Histogram();

        /** Parameters constructor */
        Histogram(const NumericalScalar first,
                  const HistogramPairCollection & collection);


        /** Comparison operator */
        Bool operator ==(const Histogram & other) const;

        /** String converter */
        String __repr__() const;
        String __str__(const String & offset = "") const;



        /* Interface inherited from Distribution */

        /** Virtual constructor */
        virtual Histogram * clone() const;

        /** Get one realization of the Histogram distribution */
        NumericalPoint getRealization() const;

        /** Get the DDF of the Histogram distribution */
        using NonEllipticalDistribution::computeDDF;
        NumericalPoint computeDDF(const NumericalPoint & point) const;

        /** Get the PDF of the Histogram distribution */
        using NonEllipticalDistribution::computePDF;
        NumericalScalar computePDF(const NumericalPoint & point) const;

        /** Get the CDF of the Histogram distribution */
        using NonEllipticalDistribution::computeCDF;
        NumericalScalar computeCDF(const NumericalPoint & point,
				   const Bool tail = false) const;

        /** Get the PDFGradient of the Histogram distribution */
        NumericalPoint computePDFGradient(const NumericalPoint & point) const;

        /** Get the CDFGradient of the Histogram distribution */
        NumericalPoint computeCDFGradient(const NumericalPoint & point) const;

        /** Parameters value and description accessor */
        NumericalPointWithDescriptionCollection getParametersCollection() const;

        /* Interface specific to Histogram */

        /** First point accessor */
        void setFirst(const NumericalScalar first);
        NumericalScalar getFirst() const;

        /** Collection accessor */
        void setPairCollection(const HistogramPairCollection & collection);
        HistogramPairCollection getPairCollection() const;

        /** Draw the PDF of the Histogram using a specific presentation */
	//        using NonEllipticalDistribution::drawPDF;
        Graph drawPDF() const;

        /** Draw the PDF of the Histogram using a specific presentation */
        Graph drawPDF(const NumericalScalar xMin,
                      const NumericalScalar xMax) const;

        /** Method save() stores the object through the StorageManager */
        void save(StorageManager::Advocate & adv) const;

        /** Method load() reloads the object from the StorageManager */
        void load(StorageManager::Advocate & adv);


      protected:


      private:

        /** Compute the mean of the distribution */
        void computeMean() const;

        /** Compute the covariance of the distribution */
        void computeCovariance() const;

        /** Get the quantile of the Histogram distribution */
        NumericalScalar computeScalarQuantile(const NumericalScalar prob,
                                              const Bool tail = false,
                                              const NumericalScalar precision = DefaultQuantileEpsilon) const;

        /** Compute the numerical range of the distribution given the parameters values */
        void computeRange();

        /** The first point of the collection */
        NumericalScalar first_;

        /** The collection of pair describing the distribution */
        HistogramPairPersistentCollection collection_;

        /** The cumulated bin widths */
        NumericalPoint cumulatedWidth_;

        /** The cumulated bin surface */
        NumericalPoint cumulatedSurface_;

        /** The total surface */
        NumericalScalar surface_;

      }; /* class Histogram */


    } /* namespace Distribution */
  } /* namespace Uncertainty */
} /* namespace OpenTURNS */

#endif /* OPENTURNS_HISTOGRAM_HXX */
